#include "ti86asm.inc"
#include "prgm_lib.h"
.message Compiling a LIBRARY???? Naughty, naughty, naughty.
.error YOU SHOULDN'T BE COMPILING A COMMON LIBRARY!!!
#if (__ASMIDE_VER < 785)	;784
	.warning You are using an old version of Assembly Studio.
#endif
.org LIB_RAM
	ret
	.db $FD	;Checksum, not an instruction. VTI just jumps over it.

;============================================================
; DrawMap -- draws an 8x16 tilemap  [Assembly Coder's Zenith]
;  input: HL = pointer to tilemap to draw, bc to pointer to sprites
;  total: 34b/91529t (including ret and entire GridPutSprite)
;============================================================
DrawMap:
 ld de,$0000            ; start drawing at (0, 0)
DrawMapL:
 push hl                ; save map pointer
 push bc
 ld l,(hl)              ; get the current tile
 ld h,0                 ; clear the upper byte
 add hl,hl              ; multiply by 8 (* 2)
 add hl,hl              ; * 4
 add hl,hl              ; * 8
 pop bc
 add hl,bc              ; add togehter to get correct sprite
 push de                ; save sprite draw location
 push bc
 call GridPutSprite     ; draw the sprite
 pop bc
 pop de                 ; restore sprite draw location
 pop hl                 ; restore map pointer
 inc hl                 ; go to next tile
 inc e                  ; move location to the right
 bit 4,e                ; if the 5th bit (e = 16), we're done
 jr z,DrawMapL          ; only jump if the row isn't complete
 ld e,0                 ; draw at the left again
 inc d                  ; move location down a row
 bit 3,d                ; check if it's done (e = 8)
 ret nz                 ; return if we're done
 jr DrawMapL            ; jump back to the top of the loop

;===========================================================
; GridPutSprite:                       [routine by Dan Eble]
;  Puts an aligned sprite at (E, D), where HL -> Sprite
;===========================================================
; IN : D  = y (0-7 inclusive)
;      E  = x (0-15 inclusive)
;      HL-> sprite
;
; OUT: AF, BC, DE, HL, IX modified
; Current total : 28b/567t (not counting ret)
;===========================================================
GridPutSprite:
 push hl
 pop ix					; ix-> sprite
 srl d          		; de = 128y+x (16 bytes/row, 8 rows)
 rra
 and $80
 or e           		; add x offset (remember x <= 15)
 ld e,a
 ld hl,$fc00            ; hl-> video ram
 add hl,de              ; hl-> video ram + offset
 ld b,8                 ; initialize loop counter
 ld de,$10              ; initialize line increment
GPS_Loop
 ld a,(ix+0)    		; get byte from sprite
 ld (hl),a      		; put byte on screen
 inc ix         		; move to next byte in sprite
 add hl,de     			; move to next line on screen
 djnz GPS_Loop
 ret

Unsigned_Multiply:
	sub a          ;Flush a
	ld h,a         ;Flush h since a=0
	ld l,a         ;Flush l since a=0
	ld d,a         ;Flush d since a=0
	cp c           ;Check if multiplied by 0
Actual_Code:
	ret z          ;Return if c=0
	add hl,de      ;Adds e to hl since d=0
	dec c          ;End of loop so one step less left
	jp Actual_Code ;Start loop again

;=========================================================================

; Sprite routines by David Phillips <david@acz.org>    http://www.acz.org/

;=========================================================================

; All sprites are 8x8 masked sprites

;

; 	PutSprite      -- black/white			      (sprite, mask)

; 	PutSpriteClip  -- black/white with clipping	  (sprite, mask)

; 	GraySprite     -- grayscale				      (dark, light, mask)

; 	GraySpriteClip -- grayscale with clipping	  (dark, light, mask)

;

; in:   hl = pointer to sprite, bc = x,y location

; out:  af, bc, de, hl, ix = trash

; size: 250 bytes

;=========================================================================

VideoDraw = $FC00		;Wierd 'cause LIB is THERE!

;=========================================================================

; in: hl = 8x8 masked sprite, bc = x,y location

PutSpriteClip:
 ex de,hl							; save pointer in de

 ld hl,PutSpriteE					; need to call putsprite

 ld (ClipSpriteE),hl				; modify call address

 jr ClipSprite						; jump to clipsprite



; in: de = sprite pointer, bc = x,y location

ClipSprite:
 ld a,b								; get x coord

 cp 249								; hanging off left edge?

 jr nc,ClipSpriteL					; then clip left edge

 cp 128								; completely off right edge?

 ret nc								; then don't draw it

 jr ClipSpriteR						; clip right edge

ClipSpriteL:
 res 7,b							; draw on right edge

 dec c								; move up a row (left clipping)

 bit 7,c							; are we on the top edge?

 jr z,ClipSpriteA					; if not, skip top fix

 ld hl,RenderSpriteCV				; point to rows to draw

 inc (hl)							; increase it (decreased later)

 dec de								; decrease sprite pointer

 ld hl,-16							; need to draw up a row

 ld (ClipSpriteX),hl				; modify top row fix value

ClipSpriteA:
 ld hl,$0218						; change instruction to <jr 2>

 ld (RenderSpriteCL),hl				; modify to skip drawing left byte

 jr ClipSpriteH						; done clipping horizontal

ClipSpriteR:
 cp 120								; hanging off right edge?

 jr c,ClipSpriteH					; if not, don't clip

 ld hl,$0218						; change instruction to <jr 2>

 ld (RenderSpriteCR),hl				; modify to skip drawing right byte

ClipSpriteH:
 ld a,c								; get y coord

 cp 249								; hanging off top?

 jr nc,ClipSpriteT					; then clip the top

 cp 63								; completely off the bottom?

 jr nc,ClipSpriteF					; then don't draw, but fix everything

 jr ClipSpriteB						; clip the bottom

ClipSpriteT:
 xor a								; clear for subtract

 ld h,a								; clear high byte

 sub c								; calc bytes above screen

 ld c,h								; set y coord to 0

 ld l,a								; set low byte to bytes above screen

 add hl,de							; advance sprite pointer

 ex de,hl							; swap back into de

 jr ClipSpriteV						; finish vertical clipping

ClipSpriteB:
 sub 57								; calc bytes below screen

 jr c,ClipSpriteD					; don't clip if not below screen

 inc a								; increase bytes below screen

ClipSpriteV:
 neg								; negate so it can be subtracted

ClipSpriteZ =$+1
 ld hl,RenderSpriteCV				; point to rows to draw

 add a,(hl)							; add (subtract) from current value

 ld (hl),a							; save new value

ClipSpriteD:
 push de							; push sprite pointer

 pop ix								; pop sprite pointer in ix

 call FindAddress					; calc address and offset of sprite

ClipSpriteX =$+1
 ld de,0							; load address fix for top row

 add hl,de							; add the fix value

ClipSpriteE =$+1
 call $ffff							; call the correct sprite routine

ClipSpriteF:
 ld a,8								; normally 8 rows are drawn

 ld (RenderSpriteCV),a				; set rows to draw back

 ld hl,$a678						; normal opcode is <ld a,b \ and (hl)>

 ld (RenderSpriteCL),hl				; set instructions back

 inc l								; change <ld a,b> to <ld a,c>

 ld (RenderSpriteCR),hl				; set instructions back

 ld hl,0							; normally top row fix is 0

 ld (ClipSpriteX),hl				; set rox fix back

 ret								; return from clipping



; in: hl = 8x8 masked sprite, bc = x,y location

PutSprite:
 push hl							; push sprite pointer

 pop ix								; pop in ix

 call FindAddress					; calc address and offset

PutSpriteE:
 ld de,VideoDraw					; draw sprite to video memory

 add hl,de							; add offset to buffer start

 jr RenderSprite					; actually draw the sprite



; in: hl = 8x8 masked grayscale sprite, bc = x,y location

; in: hl = address in buffer, c = offset, ix = sprite

RenderSprite:
RenderSpriteCV =$+1

 ld b,8								; set row counter to 8 bytes

RenderSpriteL:
 push bc							; save row counter and sprite offset

 ld a,(ix+8)						; load mask for this row

RenderSpriteM =$-1
 cpl								; invert the mask

 ld b,a								; save inverted mask

 ld d,(ix)							; load sprite byte for this row

 inc ix								; advance sprite pointer

 ld a,c								; set shift counter to sprite offset

 ld c,$ff							; set all bits of right mask byte

 ld e,0								; clear right byte of sprite

 or a								; is the sprite aligned?

 jr z,RenderSpriteD					; then skip shifting it
RenderSpriteS:
 srl d								; shift left sprite byte

 rr e								; into the right sprite byte

 scf								; set carry

 rr b 								; shift left mask byte

 rr c								; into the right mask byte

 dec a								; decrease shift counter

 jr nz,RenderSpriteS				; loop until counter is 0

RenderSpriteD:
RenderSpriteCL =$
 ld a,b								; load the left sprite byte

 and (hl)							; and it with the screen

 or d								; or it with the mask

 ld (hl),a							; set screen memory to left byte

 inc hl								; increase screen pointer

RenderSpriteCR =$
 ld a,c								; load the right sprite byte

 and (hl)							; and it with the screen

 or e								; or it with the mask

 ld (hl),a							; set screen memory to left byte

 ld de,15							; next row is 15 bytes ahead

 add hl,de							; move screen pointer to next row

 pop bc								; restore loop counter and shift mask

 djnz RenderSpriteL					; loop for all bytes in sprite

 ret								; done drawing the sprite



; in: bc = x,y location

; out: hl = offset of byte, c = offset in video byte

FindAddress:
 ld a,c								; get the y coord

 add a,a							; multiply by 2

 add a,a							; again by 4

 ld l,a								; save as low byte

 ld h,0								; clear high byte

 add hl,hl							; multiply by 8

 add hl,hl							; and again by 16

 ld a,b								; now get the x coord

 and %11111000						; clear lower 3 bits

 rra								; divide by 2

 rra								; this time by 4

 rra								; and finally by 8

 or l								; combine with low byte

 ld l,a								; save it

 ld a,b								; get x coord again

 and %00000111						; clear all but last 3 bits

 ld c,a								; save sprite offset in c

 ret								; return to caller




;------------------------------------------------------------------------------

;TextIn version 1.0-- Text input that handles upper, lowercase, and numeric chars.

;by Cassady Roop

;INPUT: hl- points to string storage area.

;		b-  max number of characters to accept and store, up to 255.

;OUTPUT: string stored to desired location.

;		 string displayed at pen location on the screen.

;		 if canceled with EXIT key, returns with nz. If successful, returns z.

;------------------------------------------------------------------------------

TextIn:
	push hl

	call _getkey			;get keypress. it destroys hl.

	pop hl

	ld c, a

	cp kenter

	jp z, textin_commit		;enter was pressed

	cp kexit				;exit key?

	jr z, textin_cancel		;EXIT	

	ld a, b					;load counter value

	cp 0					;see if char allowance has maxed out

	jr z, TextIn			;too many characters, so bypass remaining input but not the enter and exit keys.

	ld a, c					;restore keycode to a

	cp kSpace				;is it space?

	jr z, textin_space		;it is a space

	push af					;preserve the keycode

	sub $1C					;k0

	jp m, TextIn_unknown	;it's not a known key if negative

	sub $0A					;k9 +1

	jp m, textin_number		;if neg, assumed to be a number key

	sub $02					;kCapA

	jp m, TextIn_unknown	;it's not a known key if negative

	sub $1A					;kCapZ +1

	jp m, textin_capletter	;if negative, then assume capital letter

	sub $1B					;kz +1

	jp m, textin_lowletter	;if neg, assume lower case letter

textin_unknown:				;go here if it is unknown
	pop af					;what gets pushed, must get popped

	jr TextIn				;must be an unknown key at this point

textin_space:
	ld a, Lspace			;char code for space

	ld (hl), a				;put into string storage

	call _vputmap			;display a space

	inc hl					;forward pointer

	dec b					;decrement counter

	jr TextIn				;re-loop

textin_number:
	pop af					;get the original keycode

	add a, 20				;keycode-->char code

	ld (hl), a				;store

	call _vputmap			;display the number

	inc hl					;inc storage pointer

	dec b					;dec counter

	jr TextIn

textin_capletter:
	pop af

	add a, 25				;keycode-->char code

	ld (hl), a				;store

	call _vputmap			;display the letter

	inc hl					;increment string pointer

	dec b					;dec counter

	jr TextIn

textin_lowletter:
	pop af

	add a, 31				;keycode-->char code

	ld (hl), a

	call _vputmap

	inc hl

	dec b

	jr TextIn

textin_commit:				;enter key pressed
	cp a
	ret						;done

textin_cancel:				;exit key pressed
	sub a					;a=0

	cp 1					;res the z flag

	ret						;return with error


;these are some search and compare routines i've developed.i just thought

;that since many people use algorithms similar to this, you could save a bit of time.

;each one specifies the necessaryinputs and outputs.  the first one is probably

;useful in final programs, while the others are more likely to be useful in development.

;if you use any of them, please give proper credit.

;

;not that a few addresses are used for scratch space in these routines, and it is not

;necessary to include them as .db's and .dw's. you can simply give them equ's

;for _textshadow or any other scratch space you'd like.  i just included them as

;the way they are in case you're using the other scratch space for something

;else.

;

;-Jonah Cohen		<ComAsYuAre@aol.com>







;this one is from On86. mostly, i just changed the comments.

;what it does is compare to data lists of the same size and

;goes to one address if they're equal, and another if they

;aren't equal.



cphlde:

	;compares hl to de

	;already in rom, but since we're swapping pages, can't

	;rely on it being there. this is identical routine



	push hl					;save hl

	or a						;reset carry flag

	sbc hl,de					;subtract de from hl and store in hl

	pop hl					;restore hl (all that was affected by subtraction was flags)

	ret						;return


DialogBox:
	push de
	push hl
;YOU have to clear the screen!
ClearPlotSScreen:	;for DialogBox and the like
;==========================================
; clear_screen by jimi (malcolmj1@juno.com)
; input:	none
: output:	all bits in graph mem reset
; destroy:	bc,de,hl
; clock cycles:	71 t-states
; size:		14 bytes
;==========================================
	ld hl,_plotSScreen
	ld de,_plotSScreen+1
	ld (hl),0
	ld bc,1024
	ldir
;<Actual code>
;Horizontal
	ld h,1
	ld bc,(256*10)+54				;(10,12)
	ld de,(256*118)+54			;(x,y)
	call _ILine
	ld h,1
	ld bc,(256*10)+46
	ld de,(256*118)+46
	call _ILine
	ld h,1
	ld bc,(256*10)+2
	ld de,(256*118)+2
	call _ILine
;Vertical
	ld h,1
	ld bc,(256*10)+2
	ld de,(256*10)+54
	call _ILine
	ld h,1
	ld bc,(256*118)+2
	ld de,(256*118)+54
	call _ILine
;</Actual Code>
	pop hl
	ld a,14
	ld (_penCol),a
	ld a,10
	ld (_penRow),a
	call _vputs

	pop hl
	ld a,17
	ld (_penCol),a
	ld a,28
	ld (_penRow),a
	call _vputs
	ld a,(Choice)
	cp 1
	jr z,DB_Choice
;<MODIFICATION>
;	call _getkey
;</MODIFICATION>

	ret
DB_Choice:
	call _getkey
	cp kexit
	jr z,DB_C_Exit
	cp kenter
	jr z,DB_C_Enter
	jr DB_Choice
DB_C_Enter:
	ld a,3	;Enter
	ld (Choice),a
	ret
DB_C_Exit:
	ld a,4	;Exit
	ld (Choice),a
	ret
Choice:
	.db 0
GetVersion:
	ld bc,LibDescrEnd-LibDescr
	ld hl,LibDescr
	ldir
	ret

LibDescr:	;Length-Indexed; I DONT WANT PEOPLE TO USE _VPUTS! IT GETS UGLY!
	.db 20,"GraphLib version 1.0"
LibDescrEnd:
ClrLCD:
;==========================================
; clear_screen by jimi (malcolmj1@juno.com)
; input:	none
: output:	all bits in video mem reset
; destroy:	bc,de,hl
; clock cycles:	58 t-states
; size:		12 bytes
;==========================================
	ld hl,$FC00
	ld de,$FC01
	ld (hl),l
	ld bc,1024
	ldir
	ret